# Devlog
This will be my first foray into [Julia](https://julialang.org/), so first of all, lets get some links going;
* [Docs](https://docs.julialang.org/en/v1/) | [Download](https://julialang.org/downloads/) | [Package _manager_](https://pkgdocs.julialang.org/v1/) | [Environments](https://pkgdocs.julialang.org/v1/environments/)
* [Package Repository Primer](https://julialang.org/packages/) | [JuliaHub](https://juliahub.com/ui/Packages) | [JuliaPackages](https://juliapackages.com/packages)
* [CLI options](https://docs.julialang.org/en/v1/manual/command-line-options/)
* [_Creating Packages_](https://pkgdocs.julialang.org/v1/creating-packages/)
* [JuliaRegistries/General: **_Where_ to actually submit a package**](https://github.com/JuliaRegistries/General)
* [JuliaRegistries/Registrator: **_How_ to actually submit a package**](https://github.com/JuliaRegistries/Registrator.jl/)
* [An example package, beautifulalgorithms](https://juliapackages.com/packages/beautifulalgorithms). | [JuMP](https://github.com/jump-dev/JuMP.jl) | [Genie](https://github.com/GenieFramework/Genie.jl)
* [TagBot](https://discourse.julialang.org/t/ann-the-tagbot-github-app-is-deprecated-in-favour-of-the-tagbot-github-action/34344) | [PackageCompiler](https://github.com/JuliaLang/PackageCompiler.jl) | [DocumentationGenerator](https://github.com/JuliaDocs/DocumentationGenerator.jl) | [Compat](https://pkgdocs.julialang.org/v1/compatibility/)
* [Documenter](https://juliadocs.github.io/Documenter.jl/stable/man/guide/) | [Documentation style](https://docs.julialang.org/en/v1/manual/documentation/) | [Examples](https://juliadocs.github.io/Documenter.jl/stable/man/examples/) | [Documenter-hosting](https://juliadocs.github.io/Documenter.jl/stable/man/hosting/) | [Doctests](https://juliadocs.github.io/Documenter.jl/stable/man/doctests/)

Once [downloaded](https://julialang.org/downloads/), let's start with [the guide for creating a package](https://pkgdocs.julialang.org/v1/creating-packages/). The [guide here](https://pkgdocs.julialang.org/v1/creating-packages/) makes significant mention to the naming expected of a package. The name Collatz would appear wholly appropriate for a package that deals with only that! From the julia folder, `pkg> generate Collatz` does the trick (with pkg version 1.7). For some reason it didn't add my email to the `Project.toml`'s author list, so added manually. However, the name of the folder is of questionable consequence. It appears very heavily customary for julia packages to be singular entites in `X.jl` repos fot a project named `X`, although all guides I can find for it go through steps where a project in a folder `X` is created as a local git repository in `~/.julia/dev/X` and set to target some `X.jl` repo. I'm _assuming_ that when submitting the package, only the name in the `Project.toml` will matter, so rather than have the code in a subfolder, I've copied the contents created by the `pkg> generate Collatz` into the `~/julia/` directory of this repo. We'll have to wait and see I guess.

Some handy blogs ~ [this one](https://syl1.gitbook.io/julia-language-a-concise-tutorial/language-core/11-developing-julia-packages) | [oddly enough, a medium article](https://medium.com/coffee-in-a-klein-bottle/developing-your-julia-package-682c1d309507) | [and this one](https://blog.jcharistech.com/2021/09/27/how-to-develop-and-publish-julia-packages-for-beginners/).

I realise now after reading a lot of the above that while julia packages have genuine CI, the CD is only "semi" automatic (so not properly _continuous_) -- that is to say, there's github actions amongs the links above for testing and building on the package, and can be incorporated as CI, but the CD step does not yet exist, at least, not **_Out of the Box_**, and not as the recommended approach. The [Registrator](https://github.com/JuliaRegistries/Registrator.jl) linked above is the recommended way of submitting pacakges to [JuliaRegistries/General](https://github.com/JuliaRegistries/General), which requires [installing the app](https://github.com/apps/juliateam-registrator/installations/new) and commenting `@JuliaRegistrator register` on a commit/branch.

Thankfully, it appears there is a way to [target a subdirectory as "the package"](https://github.com/JuliaRegistries/Registrator.jl#registering-a-package-in-a-subdirectory) to search for a `Project.toml` instead of the repo root `~/`; for example `@JuliaRegistrator register subdir=path/to/my/package`. We also need to follow the guide to add auto docs from [Documenter](https://juliadocs.github.io/Documenter.jl/stable/man/guide/) (which looks like it came from [this thread](https://github.com/JuliaDocs/Documenter.jl/issues/1413)?). The JuMP package is a good example of heavy use of docs. A few different places with examples of docs have different scopes of interpretation for whether `docs/make.jl` should include the deploy or not, but I prefer the `julia --project=some/dir -E/e "commands; to; eval;"` syntax.

I took a small break from this to put out [julia-release](https://github.com/Skenvy/julia-release), my first github action, so that I could use it for [this project's julia-build workflow](https://github.com/Skenvy/Collatz/blob/main/.github/workflows/julia-build.yaml). With that added, we've also got to add the base [LICENSE](https://github.com/Skenvy/Collatz/blob/main/LICENSE) to this directory as well, as Julia packages are implicitly just references to a sub-tree of a repository, that requires the license to be checked in in the same directory, and won't accept a superset license. This _should_ be the same license as that in the repo root, [just duplicated here](https://github.com/Skenvy/Collatz/blob/main/julia/LICENSE). Although I'd prefer to initiate the project with a semver of 0.9.0, as a prerelease of 1.0.0 before cleaning it up and adding the proper docs, the registrator bot will comment/warn if the _first_ version of a package added to General is not one of either `1.0.0`, `0.1.0`, or `0.0.1`, so the last step is to clean it up (now that all the base tests work) and add docs. Registrator claims that _it_ automatically adds docs that are described in the `~/<julia>/docs` in the standard way, so I'll set them up, not sure how to verify 100% pre release, although will make good use of an extra `Project.toml` in `~/<julia>/docs/viewer` to run `using LiveServer; serve(dir="docs/build")` as part of the local docs test (shout out to [LiveServer](https://github.com/tlienart/LiveServer.jl)).

(The license file "here" linked above wont exist until this is merged again). It appears that it is recommended to host documentation built by Documenter on a github io page, which can be automated with a deploydocs step in the `~/docs/make.jl`, although the front page of a package on something like juliahub is a copy of the README. So there needs to be some inter-referentiality between the two.
